#pragma once

struct Partition
{
    uint64_t id;
    uint64_t diskOffsetLba;
    uint64_t sizeLba;
    uint64_t volumeOffsetLba;
    uint64_t diskId;
};

typedef std::vector<Partition*> PartitionContainer;

enum ComponentLayout
{
    ClStripe = 1,
    ClConcatenated = 2,
    ClRaid = 3
};

struct Component
{
    uint64_t id;
    ComponentLayout layout;
    PartitionContainer partitions;
};

typedef std::vector<Component*> ComponentContainer;

enum VolumeType
{
    VtUnknown,
    VtSimple,
    VtSpan,
    VtStripe,
    VtMirror,
    VtRaid5
};

enum FsType
{
    FsRawOrFat16 = 0x6,
    FsNtfs = 0x7,
    FsFat32 = 0xB
};

struct Volume
{
    uint64_t id;
    VolumeType type;
    FsType fileSystem;
    uint64_t sizeLba;
    ComponentContainer components;
};

struct Disk
{
    Disk()
        : id(0)
        , path("Missing")
        , sectorSize(512)
        , logicalStartLba(0)
    {
    }

    Disk(uint64_t objectId, const std::string& guid)
        : id(objectId)
        , guid(guid)
        , path("Missing")
        , sectorSize(512)
        , logicalStartLba(0)
    {
    }

    uint64_t id;
    std::string guid;
    std::string path;
    uint32_t sectorSize;
    uint64_t logicalStartLba;
    PartitionContainer partitions;
};
