#include "stdafx.h"
#include "PhysicalDisk.h"
#include "ErrnoException.h"

#ifdef _WIN32
#define fseeko64 _fseeki64
#endif

PhysicalDisk::PhysicalDisk(const char* path)
    : m_path(path)
    , m_drive(Open())
    , m_sectorSize(GetSectorSize())
{
}

uint32_t PhysicalDisk::SectorSize()
{
    return m_sectorSize;
}

const std::string& PhysicalDisk::GetPath()
{
    return m_path;
}

uint32_t PhysicalDisk::Read(void* buffer, uint32_t length, uint64_t offset)
{
    if (fseeko64(m_drive, offset, SEEK_SET) == -1)
    {
        throw ErrnoException("Failed to seek physical disk");
    }

    uint32_t result = fread(buffer, 1, length, m_drive);
    if (result == static_cast<uint32_t>(-1))
    {
        throw ErrnoException("Failed to read physical disk");
    }

    return result;
}

FILE* PhysicalDisk::Open()
{
    FILE* drive = fopen(m_path.c_str(), "rb");
    if (!drive)
    {
        throw ErrnoException("Failed to open physical disk");
    }

    return drive;
}

uint32_t PhysicalDisk::GetSectorSize()
{
    const uint32_t defaultValue = 512;

#ifdef _WIN32
    HANDLE drive = ::CreateFileA(
        m_path.c_str(),
        GENERIC_READ,
        FILE_SHARE_READ | FILE_SHARE_WRITE,
        NULL,
        OPEN_EXISTING,
        FILE_ATTRIBUTE_DEVICE,
        NULL);

    if (drive == INVALID_HANDLE_VALUE)
    {
        std::cout << __FUNCTION__": failed to open drive. Using default sector size value" << std::endl;
        return defaultValue;
    }

    DISK_GEOMETRY geometry = {};
    DWORD ret = 0;
    BOOL res = ::DeviceIoControl(
        drive, IOCTL_DISK_GET_DRIVE_GEOMETRY, NULL, 0, &geometry, sizeof(DISK_GEOMETRY), &ret, NULL);

    ::CloseHandle(drive);

    if (!res)
    {
        std::cout << __FUNCTION__": IoControl failed. Using default sector size value" << std::endl;
        return defaultValue;
    }

    return geometry.BytesPerSector;
#else
    int drive = open(m_path.c_str(), O_RDONLY);
    if (drive == -1)
    {
        std::cout << __FUNCTION__ << ": failed to open drive. Using default sector size value" << std::endl;
        return defaultValue;
    }

    size_t blockSize = 0;
    int res = ioctl(drive, BLKSSZGET, &blockSize);
    close(drive);

    if (res == -1)
    {
        std::cout << __FUNCTION__ << ": ioctl failed. Using default sector size value" << std::endl;
        return defaultValue;
    }

    return blockSize;
#endif
}
